@@ -1,9 +1,14 @@
-# -*- coding: utf-8 -*-
-# Copyright (C) 2018-2022, earthobservations developers.
+# Copyright (C) 2018-2025, earthobservations developers.
 # Distributed under the MIT License. See LICENSE for more info.
-def read_in_chunks(file_object, chunk_size=1024):
-    """
-    Lazy function (generator) to read a file piece by piece.
+"""Input/output utilities for the wetterdienst package."""
+
+from collections.abc import Iterator
+from io import BytesIO
+
+
+def read_in_chunks(file_object: BytesIO, chunk_size: int = 1024) -> Iterator[bytes]:
+    """Lazy function (generator) to read a file piece by piece.
+
     Default chunk size: 1k.
 
     -- https://stackoverflow.com/questions/519633/lazy-method-for-reading-big-file-in-python/519653#519653
